// redirect.c - a CGI proxy to the Amazon MP3 store
//
// Authors:
//   Aaron Bockover <abockover@novell.com>
//   Gabriel Burt <gburt@novell.com>
//
// Copyright 2010 Novell, Inc.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ------------------------------------------------------------------------
//
// Provides IP-based geolocation to an appropriate Amazon MP3 storefront,
// supporting the home and search functions. Injects the Banshee Project's
// Amazon Affiliate ID into the redirected URLs. 100% of all revenue
// generated by this affiliate ID goes to the GNOME Foundation.
//
// The proxy itself is very simple and lame. It's written to be fast and
// short lived (memory is not explicitly released).

#include <GeoIP.h>
#include <glib.h>
#include <stdlib.h>

gint
main (gint argc, gchar **argv)
{
    GeoIP *geoip;
    const gchar *remote_addr;
    const gchar *path_info;
    gchar **path_parts;
    const gchar *country;
    gchar *action;
    gchar *input;
    gchar *domain;
    gchar *affiliate_code;
    gchar *home_path, *search_category;
    gchar *p;
    gchar *dest_url = NULL;
    gboolean direct_url = FALSE;

    country = NULL;
    input = NULL;
    action = NULL;
    home_path = "mp3";
    search_category = "digital-music";

    remote_addr = g_getenv ("REMOTE_ADDR");

    path_parts = g_strsplit (path_info = g_getenv ("PATH_INFO"), "/", 4);
    if (path_parts != NULL &&
        path_parts[0] != NULL &&
        (country = path_parts[1]) != NULL &&
        (action = path_parts[2]) != NULL) {
        input = path_parts[3];
    }

    if ((p = strrchr (remote_addr, ':')) != NULL) {
        remote_addr = p + 1;
    }

    if (country == NULL || strcmp (country, "geo") == 0) {
        geoip = GeoIP_new (GEOIP_STANDARD);
        country = GeoIP_country_code_by_name (geoip, remote_addr);
    }

    // We ask that no one change these affiliate codes. ALL (100%) revenue
    // generated by these affiliate IDs is sent directly to the GNOME
    // Foundation. The GNOME Foundation controls/owns these affiliate IDs.
    // Please help support Free Software through the GNOME Foundation!
    if (country == NULL || strcmp (country, "US") == 0) {
        domain = "com";
        affiliate_code = "banshee-20";
    } else if (strcmp (country, "FR") == 0) {
        domain = "fr";
        affiliate_code = "banshee-fr-21";
    } else if (strcmp (country, "UK") == 0 ||
        strcmp (country, "GB") == 0) {
        domain = "co.uk";
        affiliate_code = "banshee-uk-21";
    } else if (strcmp (country, "DE") == 0 ||
        strcmp (country, "CH") == 0 ||
        strcmp (country, "AT") == 0) {
        domain = "de";
        affiliate_code = "banshee-de-21";
    } else if (strcmp (country, "JP") == 0) {
        domain = "co.jp";
        affiliate_code = "banshee-jp-22";
    } else if (strcmp (country, "CA") == 0) {
        domain = "ca";
        affiliate_code = "banshee-ca-20";
        // Amazon.ca doesn't have an MP3 store (yet), so
        // adjust Home and Search to use the general music section
        home_path = "music";
        search_category = "popular";
    } else {
        domain = "com";
        affiliate_code = "banshee-20";
    }

    if (strcmp (action, "search") == 0) {
        dest_url = g_strdup_printf (
            "http://www.amazon.%s/s/ref=nb_sb_noss?url=search-alias%%3D%s&field-keywords=%s",
            domain, search_category, g_uri_escape_string (input, NULL, TRUE)
        );
    } else if (strcmp (action, "sign_out") == 0) {
        dest_url = g_strdup_printf (
            "http://www.amazon.%s/gp/flex/sign-out.html/ref=gno_signout?ie=UTF8&path=%%2F%s&signIn=1&useRedirectOnSuccess=1&action=sign-out",
            domain, home_path
        );
    } else if (strcmp (action, "about") == 0) {
        dest_url = g_strdup ("http://banshee.fm/about/revenue/");
        direct_url = TRUE;
    }

    if (dest_url == NULL) {
        dest_url = g_strdup_printf ("http://www.amazon.%s/%s", domain, home_path);
    }

    if (direct_url) {
        printf ("Location: %s" "\n\n", dest_url);
    } else {
        printf ("Location: http://www.amazon.%s/gp/redirect.html?ie=UTF8&location=%s&tag=%s" "\n\n",
            domain,
            g_uri_escape_string (dest_url, NULL, TRUE),
            affiliate_code
        );
    }

    return 0;
}
